from bs4 import BeautifulSoup as bs
import requests
from typing import List
import urllib

from ua_headers import ua
from .result import Result
from .error import InvalidCalculation, InvalidPhrase, InvalidLocation

class Google:
	def __request(query: str, num: str):
		url = "https://google.com/search?q="+urllib.parse.quote(query)+"&num="+str(num)
		r = requests.get(url, headers={'User-Agent': ua.linux()})
		return bs(r.text, 'html.parser')

	@classmethod
	def search(self, query: str, num="25"):
		req = self.__request(query, "&num="+num)
		res = req.find_all("div", class_="kCrYT")
		reslist = list()
		for i in res:
			if i.a is not None and i.a['href'].startswith("/url"):
				link = urllib.parse.unquote(i.a['href']).split('?q=')[1].split('&sa=')[0]
				children = i.find_all("span", class_="XLloXe AP7Wnd")
				title = i.a.h3.div.text if i.a.h3 is not None else None
				domain = i.a.div.text if i.a.div is not None else None
				kwargs = {
					'link': link,
					'title': title,
					'domain': domain,
				}
				reslist.append(Result(**kwargs))
		return reslist

# Author: medjed
# Modifined: MishaKorzhik_He1Zen



from typing import Optional

class Result:
	'''Result returned from google.com'''
	def __init__(self, **kwargs):
		
		self.title: Optional[str] = kwargs.get('title') #: The title of a result
		self.link: Optional[str] = kwargs.get('link') #: The link of a result
		self.domain: Optional[str] = kwargs.get('domain') #: The domain of a result
		self.answer: Optional[str] = kwargs.get('answer') #: The math answer of a result
		self.question: Optional[str] = kwargs.get('question') #: The parsed math question of a result
		self.phrase: Optional[str] = kwargs.get('phrase') #: The parsed phrase of a result
		self.pronunciation: Optional[str] = kwargs.get('pronun') #: The pronunciation of a result
		self.type: Optional[str] = kwargs.get('type') #: The type of a phrase of a result
		self.meaning: Optional[str] = kwargs.get('meaning') # The meaning of a result
		self.weather: Optional[str] = kwargs.get('weather') # The weather of the location
		self.temperature: Optional[str] = kwargs.get('temp') # The temperature of the location


class GoogleUtilsError(Exception):
	pass

class InvalidCalculation(GoogleUtilsError):
	def __init__(self, calc: str):
		message: str = f'Invalid calculation: "{calc}"'
		super().__init__(message)

class InvalidPhrase(GoogleUtilsError):
	def __init__(self, def_: str):
		message = f'Unknown phrase: "{def_}"'
		super().__init__(message)

class InvalidLocation(GoogleUtilsError):
	def __init__(self, location: str):
		message = f'Unknown location: "{location}"'
		super().__init__(message)